/****************************************************************************
* 	Ox programm by Christian	 											*
*																			*
*	Calculates disaggregated time series following the Chow-Lin approach	*
*	Uses a new estimation technique.										*
*																			*
*	Please quote as follows: 												*
*	Testing temporal disaggregation,										*
*	Swiss Institute for Business Cycle Research, Working Paper 134, 2006.	*
*	URL: http://ssrn.com/abstract=910885									*
*																			*
****************************************************************************/

#include <oxstd.h> // include the Ox standard library header
#include <oxdraw.h> // include the Ox graphics library 
#import <maximize>
#import <MyCLArfima>
#import <simula>

#include <MyPcfiml.ox>
#include <Quartalisierung.ox>
#include <ChowLin.ox>
#include <QuartalisierungForecast.ox>

MyAnalyse(quelle, ziel, sofs, eofs, mMod);


/* ********************************************************************* */
/* ********** Model specification section ****************************** */
// Basic modelling information
	const decl m_mLf   = 1; 	// low frequency = s_mLf periods per year
	const decl m_mHf   = 4;		// high frequency = s_mHf periods per year
	const decl mMtrend = 1; 	// 0:= no trend, 1:= trend
	const decl mNfcst  = 2;		// Noumber of forecasts
	const decl mGraphs = 0;		// 0:= no graphical output, 1:= with graphics
	const decl mSeason = 0;		// 0:= no seasonal dummies, 1:= with seasonal dummies
/* ********************************************************************* */



main()
{
	decl quelle, ziel, sofs, eofs, sPfad, sDatei, sMD, mBranch, mQuant, mMod, mInfo;
	decl mMspec;
	sofs = unit(3); 
	eofs = unit(3); 



/* ********************************************************************* */
/* ************************ Data input section ************************* */
/* ********************************************************************* */

// 1.   Define main directory e.g.: "c:/temp/GiveWin/", called MainDir henceforth
// will be read in MyPath, approx line 291 in Quartalisierung.ox
sMD = "c:/temp/GiveWin/";
						// MUST: subdirectories INPUT and OUTPUT have to exist
						// SubDir INPUT: contains reference series (i.e. high frequency data)
						// Make sure that in INPUT the subdirectories
						// ANNUAL, QUARTERLY, MONTHLY, SEMIANNUAL exist too.
						// Make sure that in OUTPUT the subdirectories
						// QUARTERLY, MONTHLY, SEMIANNUAL exist too.


// 2. Description of the data input
// Edit the entries whenever necessary when reading in the data !
	//start of sample
	sofs[0][] = <1990, 1, m_mHf>;						  /* high frequency (desaggregate data; =related series) */
	sofs[1][] = <1990, -1, m_mLf>;						  /* low frequency (aggregate data) */

	//end of sample
	eofs[0][] = <2007, 4, m_mHf>;						  /* high frequency (desaggregate data; =related series) */
	eofs[1][] = <2007, -1, m_mLf>;						  /* low frequency (aggregate data) */

// NOTE: The -1 is necessary whenever annual data is referred to.
// Otherwise indicate the subperiod of the respective year.

																			   
// 3.  Data input for high frequency data (the series used as reference series)
	sDatei = "industry_q";
//	sDatei = "GDPNOM_q";
//	sDatei = "GDPNOM_m";


// 4.  Data input for low frequency data (the series to be temporarily disaggregated)
	sDatei = {sDatei, "industry_a"};
//	sDatei = {sDatei, "ANNUAL_GDP_NOMINAL"};

/* ********************************************************************* */
/* ******************* END of Data Input section *********************** */
/* * Do not edit the follwing unless you know ************************** */
/* * that you knwo what you are doing. ********************************* */
/* ********************************************************************* */


mMod = 0; // Model specification for supplementary analysis (0: OLS, >= 1: ARMA(0,1) )  

 quelle = MyPathNew(sDatei[0], sMD, sofs[0][]); // related series
 quelle = {quelle, MyPathNew(sDatei[1], sMD, sofs[1][])};	   // add aggregated series
 ziel = {quelle[0][1], quelle[1][1]};

print(quelle, ziel); 
 
 MyAnalyse(quelle, ziel, sofs, eofs, mMod); // calls ADF analysis if data is at quarterly frequency
										// or ADF a n d cointegration analysis if annual frequency
																	
} // End of main programm


MyAnalyse(quelle, ziel, sofs, eofs, mMod)
	{
//		MyChowLin(quelle, ziel, sofs, eofs, <1, 2, 0, 0>); // calls Chow&Lin Disaggregation procedure
		MyChowLin(quelle, ziel, sofs, eofs, <mMtrend, mNfcst, mGraphs, (mSeason*m_mHf)>); // calls Chow&Lin Disaggregation procedure
														// < Trend(=1), no. out-of-sample-forecsts, Graph(if !=0), seasonal dummies? >
    }
/* ******************************************************** */


MyAllADF(quelle, ziel, sofs, eofs, mTyn)
	{
	decl dbase, Testdats, YY, XX, NAME, j, jmax, NAMEj;
	decl Cires,  sT;

	if (mTyn == 0)
	{
		NAMEj = " WITHOUT ";
		sT = "nT";
	}
	else
	{
		NAMEj = " WITH ";
		sT = "wT";
	}
	readout(quelle, (ziel~"ADF"~sT~".txt"));		 // initialises output file
	
	dbase = new PcFiml();
	dbase.LoadXls(quelle);
	NAME = dbase->GetAllNames();
	jmax = rows(NAME);
	decl ADFAns = zeros(jmax,1);
	decl ergebnis = zeros(jmax,7);

	dbase.Deterministic(TRUE);

	decl mSampl, mT, mADFmaxl;
    mSampl = frequenz(sofs, eofs);
	mT = mSampl[4];

	j = 0;
	while (j<(jmax))
	{
	//  Supplementary ADF Test
	print("\n ADF-Test fr ", NAME[j], " \n");

	decl mj = j|mTyn;
	decl adfreslt = MyADF(quelle, mj,  sofs, eofs);	 // mj: 1. No. of Var, 2. mit Trend?
	decl adfres = adfreslt[0];
	decl mLag = adfreslt[1];
	ergebnis[j][] = adfres[0]~mLag[0]~adfres[1]~mLag[1]~adfres[2]~mLag[2]~adfreslt[2];

	j++; // End of Loop
	}
			format(600);
			sprintbuffer(20480);
			decl sample;
			sample = dbase.GetSample();

			print("\nADF Test for the Variables Model", NAMEj, " trend for ", sample, "\n",
			"%r", NAME, //CiAns
			"%cf", {"%d2.0g",  "%d2.2g", "%d2.2g", "%d2.2g", "%d2.2g"},
			"%c",  {"Level"~  "\t"~     "lags"~   "\t"~      "1stD."~  "\t"~     "lags"~   "\t"~    "2ndD."~  "\t"~     "lags"~  "\t"~     "Obs"  },
			"%cf", {"%#7.2g",  "%5.2g", "%9.2g", "%7.0g",   "%5.4g"  },
			ergebnis,
			"\nThese results (without header) are to be found at:\n", ziel~"ADF"~sT~".dat",
			"\nThis file: ", ziel~"ADF"~sT~"nT.txt"); 		  // ResCoeff %#d
			//	decl ergebnis; d: integer, g: double, "%#13.5g"
	ergebnis = sprint(
			"%r", NAME,	//CiAns
			"%cf", {"%d2.0g",  "%d2.2g", "%d2.2g", "%d2.2g", "%d2.2g"},
			"%c",  {"Level"~  "\t"~     "lags"~   "\t"~      "1stD."~  "\t"~     "lags"~   "\t"~    "2ndD."~  "\t"~     "lags"~  "\t"~     "Obs"  },
			"%cf", {"%#7.2g",  "%5.2g", "%9.2g", "%7.0g",   "%5.4g"  },
			ergebnis); 		  // ResCoeff %#d

	decl file = fopen((ziel~sT~".dat"), "w");
	fprint(file, ergebnis);
	fclose(file);		

	ergebnis = sprint("\nADF Test for the Variables Model", NAMEj, " trend for ", sample, "\n",
			"%r", NAME, //CiAns
			"%cf", {"%d2.0g",  "%d2.2g", "%d2.2g", "%d2.2g", "%d2.2g"},
			"%c",  {"Level"~  "\t"~     "lags"~   "\t"~      "1stD."~  "\t"~     "lags"~   "\t"~    "2ndD."~  "\t"~     "lags"~  "\t"~     "Obs"  },
			"%cf", {"%#7.2g",  "%5.2g", "%9.2g", "%7.0g",   "%5.4g"  },
			ergebnis,
			"\nThese results (without header) are to be found at:\n", ziel~"ADF"~sT~".dat",
			"\nThis file: ", ziel~"ADF"~sT~".txt"); 		  // ResCoeff %#d

	file = fopen((ziel~"ADF"~sT~".txt"), "a");
	fprint(file, ergebnis);
	fclose(file);		

	delete dbase;

	}
